"""Posix concurrent rotate file logging handler."""

import fcntl
import logging.handlers
import multiprocessing


class Flock(object):
    """Use flock for locking file."""

    def __init__(self, lock_filename):
        self.lock_filename = lock_filename
        self._lock = None

    def acquire(self, block=False):
        if self._lock is None:
            self._lock = open(self.lock_filename, "a")
        if block:
            fcntl.flock(self._lock, fcntl.LOCK_EX)
        else:
            try:
                fcntl.flock(self._lock, fcntl.LOCK_EX | fcntl.LOCK_NB)
            except IOError:
                return False
            return True

    def release(self):
        if self._lock is not None:
            try:
                fcntl.flock(self._lock, fcntl.LOCK_UN)
            except ValueError:
                pass


class Lockf(object):
    """Use lockf for locking file."""

    def __init__(self, lock_filename):
        self.lock_filename = lock_filename
        self._lock = None

    def acquire(self, block=False):
        if self._lock is None:
            self._lock = open(self.lock_filename, "a")
        if block:
            fcntl.lockf(self._lock, fcntl.LOCK_EX)
        else:
            try:
                fcntl.lockf(self._lock, fcntl.LOCK_EX | fcntl.LOCK_NB)
            except IOError:
                return False
            return True

    def release(self):
        if self._lock is not None:
            try:
                fcntl.lockf(self._lock, fcntl.LOCK_UN)
            except ValueError:
                pass


class PLogHandler(logging.handlers.RotatingFileHandler):
    """Uses unix file locks for concurrent rollovering files."""

    def __init__(self, filename, mode="a", maxBytes=0, backupCount=0, encoding=None,
        delay=0, rotlock=None):
        """ConcurrentRotatingFileHandler has one additinal keyword argument
        comparing to RotatingFileHandler - rotlock.

        :param rotlock: object used for locking, Lockf can be used
        :type rotlock: should provide acquire(block=True) and release methods
        """
        logging.handlers.RotatingFileHandler.__init__(self, filename,
            mode=mode, maxBytes=maxBytes, backupCount=backupCount,
            encoding=encoding, delay=delay)
        self._rotlock = rotlock
        if self._rotlock is None:
            self._rotlock = multiprocessing.Lock()

    def rotlock(self, block=True):
        return self._rotlock.acquire(block)

    def rotunlock(self):
        return self._rotlock.release()

    def acquire(self):
        logging.handlers.RotatingFileHandler.acquire(self)
        self.rotlock()

    def release(self):
        self.rotunlock()
        logging.handlers.RotatingFileHandler.release(self)

    def _open(self):
        """Open the current base file with buffering disabled."""
        # disable buffering to make fwrite do 1 write per call (to avoid "concurrency")
        stream = open(self.baseFilename, self.mode, 0)
        return stream


ConcurrentRotatingFileHandler = PLogHandler



import os
import sys
from setuptools import setup


PACKAGE_PATH = os.path.abspath(os.path.dirname(__file__))

def get_description():
    with open(os.path.join(PACKAGE_PATH, "README.rst")) as readme:
        return readme.read()

setup(
    name="ploghandler",
    version="0.2",
    author="Vadim Radovel",
    author_email="vadim@radovel.ru",
    description="Provides concurrent rotating file handler for posix-compatible OSes.",
    license="GPLv3",
    url="https://github.com/NightBlues/ploghandler",
    py_modules=["ploghandler"],
    test_suite="test",
    long_description=get_description(),
    platforms = ["posix"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Operating System :: Unix",
        "Programming Language :: Python :: 2",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: System :: Logging"
    ]
)



import unittest
import tempfile
from multiprocessing import Process, Queue

class PLogHandlerTest(unittest.TestCase):

	def test_import(self):
		import ploghandler

	def test_lockf_nonblock(self):
		from ploghandler import Lockf
		lock_file, lock_filename = tempfile.mkstemp(".lock")

		def locker(filename, cmd_queue, result_queue):
			lock = Lockf(lock_filename)
			while True:
				cmd = cmd_queue.get()
				if cmd == "quit":
					break
				elif cmd == "lock":
					result_queue.put(lock.acquire(block=False))
				elif cmd == "unlock":
					lock.release()

		p1_c, p1_r = Queue(), Queue()
		p1 = Process(target=locker, args=(lock_filename, p1_c, p1_r))
		p2_c, p2_r = Queue(), Queue()
		p2 = Process(target=locker, args=(lock_filename, p2_c, p2_r))

		p1.start()
		p2.start()
		
		p1_c.put("lock")
		self.assertTrue(p1_r.get())
		p2_c.put("lock")
		self.assertFalse(p2_r.get())
		p1_c.put("unlock")
		p2_c.put("lock")
		self.assertTrue(p2_r.get())
		p2_c.put("unlock")
